@@ -154,6 +154,8 @@ class DB2Reflector(BaseReflector):
       Column("SCALE", sa_types.Integer, key="scale"),
       Column("DEFAULT", CoerceUnicode, key="defaultval"),
       Column("NULLS", CoerceUnicode, key="nullable"),
+      Column("KEYSEQ", CoerceUnicode, key="keyseq"),
+      Column("PARTKEYSEQ", CoerceUnicode, key="partkeyseq"),
       Column("IDENTITY", CoerceUnicode, key="identity"),
       Column("GENERATED", CoerceUnicode, key="generated"),
       Column("REMARKS", CoerceUnicode, key="remarks"),
@@ -326,15 +328,15 @@ class DB2Reflector(BaseReflector):
     def get_primary_keys(self, connection, table_name, schema=None, **kw):
         current_schema = self.denormalize_name(schema or self.default_schema_name)
         table_name = self.denormalize_name(table_name)
-        sysindexes = self.sys_indexes
+        syscols = self.sys_columns
         col_finder = re.compile("(\w+)")
-        query = sql.select([sysindexes.c.colnames],
+        query = sql.select([syscols.c.colname],
               sql.and_(
-                  sysindexes.c.tabschema == current_schema,
-                  sysindexes.c.tabname == table_name,
-                  sysindexes.c.uniquerule == 'P'
+                  syscols.c.tabschema == current_schema,
+                  syscols.c.tabname == table_name,
+                  syscols.c.keyseq > 0
                 ),
-              order_by=[sysindexes.c.tabschema, sysindexes.c.tabname]
+              order_by=[syscols.c.tabschema, syscols.c.tabname]
             )
         pk_columns = []
         for r in connection.execute(query):
@@ -441,6 +443,8 @@ class DB2Reflector(BaseReflector):
             if r[2] != 'P':
                 if r[2] == 'U' and r[3] != 0:
                     continue
+                if 'sqlnotapplicable' in r[1].lower():
+                    continue
                 indexes.append({
                         'name': self.normalize_name(r[0]),
                         'column_names': [self.normalize_name(col)
@@ -608,11 +612,10 @@ class AS400Reflector(BaseReflector):
     @reflection.cache
     def get_schema_names(self, connection, **kw):
         sysschema = self.sys_schemas
-        query = sql.select([sysschema.c.schemaname],
-                sql.not_(sysschema.c.schemaname.like('SYS%')),
-                sql.not_(sysschema.c.schemaname.like('Q%')),
-                order_by=[sysschema.c.schemaname]
-        )
+        query = sql.select([sysschema.c.schemaname]).\
+                where(~sysschema.c.schemaname.like(unicode('Q%'))).\
+                where(~sysschema.c.schemaname.like(unicode('SYS%'))).\
+                order_by(sysschema.c.schemaname)
         return [self.normalize_name(r[0]) for r in connection.execute(query)]
 
     # Retrieves a list of table names for a given schema
@@ -621,7 +624,7 @@ class AS400Reflector(BaseReflector):
         current_schema = self.denormalize_name(schema or self.default_schema_name)
         systbl = self.sys_tables
         query = sql.select([systbl.c.tabname]).\
-                where(systbl.c.tabtype == 'T').\
+                where(systbl.c.tabtype == unicode('T')).\
                 where(systbl.c.tabschema == current_schema).\
                 order_by(systbl.c.tabname)
         return [self.normalize_name(r[0]) for r in connection.execute(query)]
@@ -686,9 +689,9 @@ class AS400Reflector(BaseReflector):
             sa_columns.append({
                     'name': self.normalize_name(r[0]),
                     'type': coltype,
-                    'nullable': r[3] == 'Y',
+                    'nullable': r[3] == unicode('Y'),
                     'default': r[2],
-                    'autoincrement': (r[6] == 'YES') and (r[7] != None),
+                    'autoincrement': (r[6] == unicode('YES')) and (r[7] != None),
                     'comment': r[8] or None,
                 })
         return sa_columns
@@ -732,7 +735,7 @@ class AS400Reflector(BaseReflector):
                     syskeyconst.c.conname == sysconst.c.conname,
                     sysconst.c.tabschema == current_schema,
                     sysconst.c.tabname == table_name,
-                    sysconst.c.contype == 'PRIMARY KEY'
+                    sysconst.c.contype == unicode('PRIMARY KEY')
             ), order_by=[syskeyconst.c.colno])
 
         return [self.normalize_name(key[0])
@@ -782,6 +785,7 @@ class AS400Reflector(BaseReflector):
         current_schema = self.denormalize_name(
                                     schema or self.default_schema_name)
         table_name = self.denormalize_name(table_name)
+        
         sysidx = self.sys_indexes
         syskey = self.sys_keys
 
@@ -802,7 +806,7 @@ class AS400Reflector(BaseReflector):
                 indexes[key] = {
                                 'name': self.normalize_name(r[0]),
                                 'column_names': [self.normalize_name(r[2])],
-                                'unique': r[1] == 'Y'
+                                'unique': r[1] == unicode('Y')
                         }
         return [value for key, value in indexes.items()]
 
@@ -1204,4 +1208,3 @@ class OS390Reflector(BaseReflector):
                         'column_names': [self.normalize_name(r[1])],
                     })
         return uniqueConsts
-
